package org.foxarmy.barcodescannerforemployees.activities

import android.Manifest
import android.content.ContentValues
import android.os.Build
import android.os.Bundle
import android.provider.BaseColumns
import android.util.Log
import android.widget.*
import androidx.activity.result.contract.ActivityResultContracts
import androidx.annotation.RequiresApi
import androidx.appcompat.app.AppCompatActivity
import com.google.mlkit.vision.barcode.common.Barcode
import com.google.mlkit.vision.codescanner.GmsBarcodeScannerOptions
import com.google.mlkit.vision.codescanner.GmsBarcodeScanning
import org.foxarmy.barcodescannerforemployees.*
import org.foxarmy.barcodescannerforemployees.databinding.ActivityAddProductBinding
import java.io.File
import java.nio.file.Files
import java.nio.file.StandardCopyOption

class AddProductActivity : AppCompatActivity() {
    private lateinit var imageView: ImageView

    private lateinit var saveButton: Button
    private lateinit var takePictureButton: Button
    private lateinit var scanButton: Button

    private lateinit var productNameText: TextView
    private lateinit var netWeightText: TextView

    private lateinit var categorySpinner: Spinner

    private var product: AbstractProduct? = null
    private lateinit var pictureFile: File
    private lateinit var picturesPath: File
    private lateinit var binding: ActivityAddProductBinding

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        setContentView(R.layout.fragment_add_product)

        val extras = intent.extras
        product = extras!!.get("updatingObject") as AbstractProduct?

        picturesPath = File(filesDir, "pictures")
        val thumbnailsDir = File(cacheDir, "thumbnails")
        thumbnailsDir.mkdirs()
        picturesPath.mkdirs()
        imageView = findViewById(R.id.imageView)

        saveButton = findViewById(R.id.saveButton)
        takePictureButton = findViewById(R.id.takePictureButton)
        scanButton = findViewById(R.id.scan_button)

        productNameText = findViewById(R.id.productName)
        netWeightText = findViewById(R.id.netWeight)

        categorySpinner = findViewById(R.id.categorySpinner)

        fillupCategorySpinner()

        if (product != null) {
            val imageThumbnailUri = getImageUri(this, File(thumbnailsDir, "${product!!.imageHash}.webp"))
            pictureFile = File(picturesPath, "${product!!.imageHash}.png]")
            imageView.setImageURI(imageThumbnailUri)
            imageView.rotation = 90f
            productNameText.text = product!!.name
            netWeightText.text = product!!.netWeight.toString()
            categorySpinner.setSelection(product!!.category)
        }

        saveButton.setOnClickListener {
            val productName = productNameText.text.toString()
            val netWeight = netWeightText.text
            if (!this::pictureFile.isInitialized && !pictureFile.exists()) {

                Toast.makeText(this, "Please, make a picture of a product!", Toast.LENGTH_SHORT).show()
                return@setOnClickListener
            }

            if (productName == "") {
                Toast.makeText(this, "Please, write a name of a product!", Toast.LENGTH_SHORT).show()
                return@setOnClickListener
            }
            if (netWeight.toString() == "" || netWeight.toString().toDoubleOrNull() == null) {
                Toast.makeText(this, "Please, write a valid net weight of a product!", Toast.LENGTH_SHORT).show()
            }

            val db = DBStorageController(this).writableDatabase
            val values = ContentValues().apply {
                put(ProductContract.ProductEntry.PRODUCT_NAME, productName)
                put(ProductContract.ProductEntry.PRODUCT_NET_WEIGHT, netWeight.toString())
                put(ProductContract.ProductEntry.IMAGE_FILENAME, pictureFile.nameWithoutExtension)
                put(ProductContract.ProductEntry.CATEGORY, categorySpinner.selectedItemPosition)
            }

            if (product == null) {
                db.insert(ProductContract.ProductEntry.TABLE_NAME, null, values)
            } else {
                db.update(ProductContract.ProductEntry.TABLE_NAME, values, "${BaseColumns._ID} = ?", arrayOf(product!!.id.toString()))
            }

            finish()
        }

        takePictureButton.setOnClickListener {
            requestPermissionLauncher.launch(Manifest.permission.CAMERA)
        }

        scanButton.setOnClickListener {
            val options = GmsBarcodeScannerOptions.Builder()
                .setBarcodeFormats(
                    Barcode.FORMAT_EAN_13
                )
                .build()
            val scanner = GmsBarcodeScanning.getClient(this)
            scanner.startScan()
                .addOnSuccessListener { barcode ->
                    productNameText.setText(barcode.rawValue)
                }
                .addOnFailureListener { e ->
                    Toast.makeText(
                        this,
                        "Failed to scan barcode. Please, try again or enter data manually",
                        Toast.LENGTH_LONG
                    ).show()
                }
        }

//        binding = ActivityAddProductBinding.inflate(layoutInflater)

//        setContentView(binding.root)

    }

    fun fillupCategorySpinner() {
        val db = DBStorageController(this).readableDatabase

        val categories = mutableListOf("")

        val projection = arrayOf(
            CategoriesContract.CategoryEntry.CATEGORY_NAME
        )

        val cursor = db.query(CategoriesContract.CategoryEntry.TABLE_NAME, projection, null, null, null, null, BaseColumns._ID+" ASC")

        with (cursor) {
            while (moveToNext()) {
                categories.add(getString(getColumnIndexOrThrow(CategoriesContract.CategoryEntry.CATEGORY_NAME)))
            }
        }

        val arrayAdapter = ArrayAdapter<String>(this, androidx.appcompat.R.layout.support_simple_spinner_dropdown_item, categories)
        arrayAdapter.setDropDownViewResource(androidx.appcompat.R.layout.support_simple_spinner_dropdown_item)
        categorySpinner.adapter = arrayAdapter
    }

    @RequiresApi(Build.VERSION_CODES.R)
    val takePicture = registerForActivityResult(ActivityResultContracts.TakePicture()) { success: Boolean ->
        if (success) {
            //Move picture to a proper directory according to its calculated hash
            val tempfile = File(filesDir, "image.png")
            val imageContent = tempfile.inputStream().readBytes()
            val imageHash = imageContent.toString(Charsets.UTF_8).md5()

            pictureFile = File(picturesPath, "$imageHash.png")
            Files.move(tempfile.toPath(), pictureFile.toPath(), StandardCopyOption.REPLACE_EXISTING)
            tempfile.delete()
            generateThumbnailForImage(this, imageHash)

            imageView.setImageURI(getImageUri(this, pictureFile))
        } else {
            Log.e("QWERTYUIOP", "Cannot save a picture")
        }
    }

    @RequiresApi(Build.VERSION_CODES.R)
    fun getPicture () {
        //Saving picture to a temp file for further hash calculation and moving to a proper directory
        val imageFile = File(this.filesDir, "image.png")
        val imageUri = getImageUri(this, imageFile)
        if (imageUri != null) {
            takePicture.launch(imageUri)
        }
    }

    @RequiresApi(Build.VERSION_CODES.R)
    val requestPermissionLauncher =
        registerForActivityResult(
            ActivityResultContracts.RequestPermission()
        ) { isGranted: Boolean ->
            if (isGranted) {
                getPicture()
            } else {
                Toast.makeText(this, "I need permission in order to take a picture", Toast.LENGTH_LONG).show()
            }
        }
}